using System;
using NUnit.Framework;
using PkmnLibSharp.Battling;
using PkmnLibSharp.Library;

namespace PkmnLibSharpTests.Battling
{
    public class PokemonBuilderTests
    {
        [Test]
        public void SimpleBuildPokemon()
        {
            var lib = BattleLibraryHelper.GetLibrary();
            var pokemon = new PokemonBuilder(lib, "testSpecies", 50)
                .Build();
            Assert.AreEqual("testSpecies", pokemon.Value.Species.Name);
            Assert.AreEqual(50, pokemon.Value.Level);
            Assert.AreEqual("default", pokemon.Value.Forme.Name);
            pokemon.Dispose();
        }
        
        [Test]
        public void BuildPokemonWithNickname()
        {
            var lib = BattleLibraryHelper.GetLibrary();
            var pokemon = new PokemonBuilder(lib, "testSpecies", 50)
                {
                    Nickname = "cuteNickname"
                }.Build();
            Assert.AreEqual("cuteNickname", pokemon.Value.Nickname);
            pokemon.Dispose();
        }

        [Test]
        public void BuildPokemonWithGender()
        {
            var lib = BattleLibraryHelper.GetLibrary();
            var pokemon = new PokemonBuilder(lib, "testSpecies", 50)
                {
                    Gender = Gender.Female
                }
                .Build();
            Assert.AreEqual(Gender.Female, pokemon.Value.Gender);
            pokemon.Dispose();
        }

        [Test]
        public void BuildPokemonWithMoves()
        {
            var lib = BattleLibraryHelper.GetLibrary();
            var pokemon = new PokemonBuilder(lib, "testSpecies", 50)
                .LearnMove("testMove", MoveLearnMethod.Unknown)
                .LearnMove("testMove2", MoveLearnMethod.Level)
                .Build();
            Assert.AreEqual("testMove", pokemon.Value.Moves[0].Move.Name);
            Assert.AreEqual("testMove2", pokemon.Value.Moves[1].Move.Name);
            pokemon.Dispose();
        }

    }
}